/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_IPAssignmentSettingData.h"

#include "network.h"
#include "connection.h"

static const CMPIBroker* _cb = NULL;

static void Cura_IPAssignmentSettingDataInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_IPAssignmentSettingDataCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPAssignmentSettingDataEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_IPAssignmentSettingDataEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    Connection *connection;
    const Connections *connections = network_get_connections(network);
    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        Cura_IPAssignmentSettingData w;
        Cura_IPAssignmentSettingData_Init(&w, _cb, ns);
        Cura_IPAssignmentSettingData_Set_ElementName(&w, connection_get_id(connection));
        Cura_IPAssignmentSettingData_Set_Caption(&w, connection_get_name(connection));
        Cura_IPAssignmentSettingData_Set_InstanceID(&w, connection_get_id(connection));

        KReturnInstance(cr, w);
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPAssignmentSettingDataGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_IPAssignmentSettingDataCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_IPAssignmentSettingDataModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_IPAssignmentSettingDataDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    Network *network = mi->hdl;

    Cura_IPAssignmentSettingData w;
    Cura_IPAssignmentSettingData_InitFromObjectPath(&w, _cb, cop);
    const char *uuid = w.InstanceID.chars;

    const Connections *connections = network_get_connections(network);
    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        if (strcmp(connection_get_id(connections_index(connections, i)), uuid) == 0) {
            int rc = network_delete_connection(network, connections_index(connections, i));

            if (rc != 0) {
                CMReturn(CMPI_RC_ERR_FAILED);
            }
            CMReturn(CMPI_RC_OK);
        }
    }
    CMReturn(CMPI_RC_ERR_FAILED);
}

static CMPIStatus Cura_IPAssignmentSettingDataExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_IPAssignmentSettingData,
    Cura_IPAssignmentSettingData,
    _cb,
    Cura_IPAssignmentSettingDataInitialize(&mi))

static void Cura_IPAssignmentSettingDataMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}


static CMPIStatus Cura_IPAssignmentSettingDataMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_IPAssignmentSettingDataInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_IPAssignmentSettingData_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_IPAssignmentSettingData,
    Cura_IPAssignmentSettingData,
    _cb,
    Cura_IPAssignmentSettingDataMethodInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_IPAssignmentSettingData",
    "Cura_IPAssignmentSettingData",
    "instance method")
