/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_RemoteAccessAvailableToElement.h"
#include <Cura_RemoteServiceAccessPoint.h>
#include "network.h"
#include "activeconnection.h"
#include "connection.h"
#include "setting.h"
#include "endpoint.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void Cura_RemoteAccessAvailableToElementInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    unsigned int j;
    const Ports *ports = network_get_ports(network);
    Port *port;
    Endpoint *endpoint;

    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        for (j = 0; j < endpoints_length(port_get_endpoints(port)); ++j) {
            endpoint = endpoints_index(port_get_endpoints(port), j);

            if (endpoint_get_type(endpoint) == Unknown || (endpoint_get_ipv4gateway(endpoint) == NULL && endpoint_get_ipv6gateway(endpoint) == NULL)) {
                continue;
            }

            CMPIObjectPath *remoteServiceAccessPointOP = endpointToCura_RemoteServiceAccessPointOP(endpoint, _cb, ns);

            CMPIObjectPath *ipProtocolEndpointOP = endpointToCura_IPProtocolEndpointRefOP(endpoint, _cb, ns);

            Cura_RemoteAccessAvailableToElement w;
            Cura_RemoteAccessAvailableToElement_Init(&w, _cb, ns);
            Cura_RemoteAccessAvailableToElement_SetObjectPath_Antecedent(&w, remoteServiceAccessPointOP);
            Cura_RemoteAccessAvailableToElement_SetObjectPath_Dependent(&w, ipProtocolEndpointOP);
            KReturnInstance(cr, w);
        }
    }

    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void Cura_RemoteAccessAvailableToElementAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Cura_RemoteAccessAvailableToElementReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_RemoteAccessAvailableToElement_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    Cura_RemoteAccessAvailableToElement,
    Cura_RemoteAccessAvailableToElement,
    _cb,
    Cura_RemoteAccessAvailableToElementInitialize(&mi))

CMAssociationMIStub(
    Cura_RemoteAccessAvailableToElement,
    Cura_RemoteAccessAvailableToElement,
    _cb,
    Cura_RemoteAccessAvailableToElementAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_RemoteAccessAvailableToElement",
    "Cura_RemoteAccessAvailableToElement",
    "instance association")
