/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <konkret/konkret.h>
#include "Cura_ServiceAffectsElement.h"
#include "network.h"
#include "port.h"
#include "endpoint.h"
#include "ref_factory.h"

static const CMPIBroker* _cb;

static void Cura_ServiceAffectsElementInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_ServiceAffectsElementCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ServiceAffectsElementEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_ServiceAffectsElementEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    CMPIObjectPath *affectingElementOP = getIPConfigurationSystemOP(_cb, ns);

    const Ports *ports = network_get_ports(network);
    const Endpoints *endpoints;
    Endpoint *endpoint;
    unsigned int j;
    for (unsigned int i = 0; i < ports_length(ports); ++i) {
        endpoints = network_get_endpoints_by_port(network, ports_index(ports, i));
        for (j = 0; j < endpoints_length(endpoints); ++j) {
            endpoint = endpoints_index(endpoints, j);

            CMPIObjectPath *affectedElementOP = endpointToCura_IPProtocolEndpointRefOP(endpoint, _cb, ns);

            Cura_ServiceAffectsElement w;
            Cura_ServiceAffectsElement_Init(&w, _cb, ns);
            Cura_ServiceAffectsElement_SetObjectPath_AffectedElement(&w, affectedElementOP);
            Cura_ServiceAffectsElement_SetObjectPath_AffectingElement(&w, affectingElementOP);

            KReturnInstance(cr, w);
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ServiceAffectsElementGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_ServiceAffectsElementCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ServiceAffectsElementModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ServiceAffectsElementDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_ServiceAffectsElementExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void Cura_ServiceAffectsElementAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_ServiceAffectsElementAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_ServiceAffectsElementAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ServiceAffectsElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus Cura_ServiceAffectsElementAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ServiceAffectsElement_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus Cura_ServiceAffectsElementReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ServiceAffectsElement_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus Cura_ServiceAffectsElementReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        Cura_ServiceAffectsElement_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    Cura_ServiceAffectsElement,
    Cura_ServiceAffectsElement,
    _cb,
    Cura_ServiceAffectsElementInitialize(&mi))

CMAssociationMIStub(
    Cura_ServiceAffectsElement,
    Cura_ServiceAffectsElement,
    _cb,
    Cura_ServiceAffectsElementAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_ServiceAffectsElement",
    "Cura_ServiceAffectsElement",
    "instance association")
