/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "Cura_StaticIPAssignmentSettingData.h"
#include "network.h"
#include "setting.h"
#include "connection.h"

static const CMPIBroker* _cb = NULL;

static void Cura_StaticIPAssignmentSettingDataInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    unsigned int j;
    Connection *connection;
    Setting *setting;
    const Connections *connections = network_get_connections(network);
    for (unsigned int i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        for (j = 0; j < settings_length(connection_get_settings(connection)); ++j) {
            setting = settings_index(connection_get_settings(connection), j);
            if (setting_get_type(setting) != SETTING_STATIC && setting_get_type(setting) != SETTING_LINK_LOCAL) {
                continue;
            }

            Cura_StaticIPAssignmentSettingData w;
            Cura_StaticIPAssignmentSettingData_Init(&w, _cb, ns);
            Cura_StaticIPAssignmentSettingData_Set_ElementName(&w, setting_get_id(setting));
            Cura_StaticIPAssignmentSettingData_Set_Caption(&w, setting_get_caption(setting));
            Cura_StaticIPAssignmentSettingData_Set_InstanceID(&w, setting_get_id(setting));
            if (setting_get_protocol_type(setting) == IPv4) {
                Cura_StaticIPAssignmentSettingData_Set_ProtocolIFType(&w, Cura_StaticIPAssignmentSettingData_ProtocolIFType_IPv4);
                Cura_StaticIPAssignmentSettingData_Set_IPv4Address(&w, setting_get_address(setting));
                Cura_StaticIPAssignmentSettingData_Set_SubnetMask(&w, prefixToMask4(setting_get_prefix(setting)));
                Cura_StaticIPAssignmentSettingData_Set_GatewayIPv4Address(&w, setting_get_default_gateway(setting));
            } else {
                Cura_StaticIPAssignmentSettingData_Set_ProtocolIFType(&w, Cura_StaticIPAssignmentSettingData_ProtocolIFType_IPv6);
                Cura_StaticIPAssignmentSettingData_Set_IPv6Address(&w, setting_get_address(setting));
                Cura_StaticIPAssignmentSettingData_Set_IPv6SubnetPrefixLength(&w, setting_get_prefix(setting));
                Cura_StaticIPAssignmentSettingData_Set_GatewayIPv6Address(&w, setting_get_default_gateway(setting));
            }

            KReturnInstance(cr, w);
        }
    }
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    Cura_StaticIPAssignmentSettingData,
    Cura_StaticIPAssignmentSettingData,
    _cb,
    Cura_StaticIPAssignmentSettingDataInitialize(&mi))

static void Cura_StaticIPAssignmentSettingDataMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus Cura_StaticIPAssignmentSettingDataInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return Cura_StaticIPAssignmentSettingData_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    Cura_StaticIPAssignmentSettingData,
    Cura_StaticIPAssignmentSettingData,
    _cb,
    Cura_StaticIPAssignmentSettingDataMethodInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "Cura_StaticIPAssignmentSettingData",
    "Cura_StaticIPAssignmentSettingData",
    "instance method")
