/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "connection.h"
#include "connection_private.h"
#include <string.h>

Connection *connection_new(const char *id, const char *name)
{
    Connection *connection = malloc(sizeof(Connection));
    connection->id = (id != NULL) ? strdup(id) : NULL;
    connection->name = (name != NULL) ? strdup(name) : NULL;
    connection->settings = NULL;
    connection->port = NULL;
    return connection;
}

const char *connection_get_id(const Connection *connection)
{
    return connection->id;
}

const char *connection_get_name(const Connection *connection)
{
    return connection->name;
}

bool connection_get_autoconnect(const Connection *connection)
{
    return connection->autoconnect;
}

const Settings *connection_get_settings(const Connection *connection)
{
    return connection->settings;
}

Port *connection_get_port(const Connection *connection)
{
    return connection->port;
}

void *connection_get_priv(const Connection *connection)
{
    return connection->priv;
}

void connection_add_setting(Connection *connection, Setting *setting)
{
    if (connection->settings == NULL) {
        connection->settings = settings_new(1);
    }
    settings_add(connection->settings, setting);
}

void connection_set_port(Connection *connection, Port *port)
{
    connection->port = port;
}

bool connection_compare(const Connection *c1, const Connection *c2)
{
    return strcmp(c1->id, c2->id) == 0;
}

void connection_free(Connection *connection)
{
    if (connection->settings != NULL) {
        settings_free(connection->settings);
    }
    if (connection->id != NULL) {
        free(connection->id);
    }
    if (connection->name != NULL) {
        free(connection->name);
    }
    free(connection);
}

LIST_IMPL(Connection, connection)
