/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "endpoint.h"
#include "endpoint_private.h"
#ifndef TESTING
#include <nm-setting-ip4-config.h>
#include <nm-setting-ip6-config.h>
#endif

Endpoint *endpoint_new_from_NMIP4Address(Port *port, NMIP4Address *address)
{
    Endpoint *e = endpoint_new(port);
    e->type = IPv4;
    e->ipv4address = ip4ToString(nm_ip4_address_get_address(address));
    e->ipv4netmask = prefixToMask4(nm_ip4_address_get_prefix(address));
    e->ipv4gateway = ip4ToString(nm_ip4_address_get_gateway(address));
    return e;
}

Endpoint *endpoint_new_from_NMIP6Address(Port *port, NMIP6Address *address)
{
    Endpoint *e = endpoint_new(port);
    e->type = IPv6;
    e->ipv6address = ip6ToString(nm_ip6_address_get_address(address));
    e->ipv6prefix = (uint8_t) nm_ip6_address_get_prefix(address);
    e->ipv6gateway = ip6ToString(nm_ip6_address_get_gateway(address));
    return e;
}
