/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "port.h"
#include "endpoint.h"
#include "port_private.h"

#include <string.h>
#include <stdio.h>

Port *port_new(void)
{
    Port *p = malloc(sizeof(Port));
    p->id = NULL;
    p->mac = NULL;
    p->priv = NULL;
    p->endpoints = NULL;
    return p;
}

void *port_get_priv(const Port *port)
{
    return port->priv;
}

const char *port_get_id(const Port *port)
{
    return port->id;
}

PortType port_get_type(const Port *port)
{
    return port->type;
}

PortState port_get_state(const Port *port)
{
    return port->state;
}

const char *port_get_mac(const Port *port)
{
    return port->mac;
}

const Endpoints *port_get_endpoints(const Port *port)
{
    return port->endpoints;
}

bool port_compare(const Port *p1, const Port *p2)
{
    return strcmp(p1->id, p2->id) == 0;
}

void port_free(Port *p)
{
    if (p->id) {
        free(p->id);
    }
    if (p->mac) {
        free(p->mac);
    }
    if (p->endpoints) {
        endpoints_free(p->endpoints);
    }
    free(p);
}

LIST_IMPL(Port, port)
