/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PORT_H
#define PORT_H

#include "globals.h"

typedef enum PortType {
    TYPE_UNKNOWN = 0,
    TYPE_ETHERNET,
    TYPE_WIFI
} PortType;

typedef enum PortState {
    STATE_UNKNOWN = 0, STATE_NA, STATE_SERVICING, STATE_STARTING,
    STATE_STOPPING, STATE_STOPPED, STATE_ABORTED, STATE_DORMANT,
    STATE_COMPLETED, STATE_MIGRATING, STATE_EMIGRATING, STATE_IMMIGRATING,
    STATE_SNAPSHOTTING, STATE_SHUTTING_DOWN, STATE_IN_TEST,
    STATE_TRANSITIONING, STATE_IN_SERVICE
} PortState;

Port *port_new(void);

void *port_get_priv(const Port *port);
const char *port_get_id(const Port *port);
PortType port_get_type(const Port *port);
PortState port_get_state(const Port *port);
const char *port_get_mac(const Port *port);
const Endpoints *port_get_endpoints(const Port *port);

bool port_compare(const Port *p1, const Port *p2);

void port_free(Port *p);

LIST_DECL(Port, port)

#endif
