/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "port_nm.h"
#include "port_private.h"

#include <string.h>

#ifndef TESTING
#include <nm-device.h>
#include <nm-device-ethernet.h>
#include <nm-device-wifi.h>
#endif

Port *port_new_from_NMDevice(NMDevice *device)
{
    Port *port = port_new();
    port->id = strdup(nm_device_get_iface(device));
    port->priv = device;
    switch (nm_device_get_device_type(device)) {
        case NM_DEVICE_TYPE_ETHERNET:
            port->type = TYPE_ETHERNET;
            port->mac = strdup(nm_device_ethernet_get_hw_address((NMDeviceEthernet *) device));
            break;
        case NM_DEVICE_TYPE_WIFI:
            port->type = TYPE_WIFI;
            port->mac = strdup(nm_device_wifi_get_hw_address((NMDeviceWifi *) device));
            break;
        case NM_DEVICE_TYPE_BT:
        case NM_DEVICE_TYPE_OLPC_MESH:
        case NM_DEVICE_TYPE_WIMAX:
        case NM_DEVICE_TYPE_MODEM:
        case NM_DEVICE_TYPE_INFINIBAND:
        case NM_DEVICE_TYPE_BOND:
        case NM_DEVICE_TYPE_VLAN:
            fprintf(stderr, "Device type is not supported yet");
            port->type = TYPE_UNKNOWN;
            port->mac = NULL;
        default:
            port->type = TYPE_UNKNOWN;
            port->mac = NULL;
    }

    port->state = port_state_from_NMDeviceState(nm_device_get_state(device));
    return port;
}

PortState port_state_from_NMDeviceState(NMDeviceState state)
{
    debug("Port state: %d", state);
    switch (state) {
        case NM_DEVICE_STATE_UNKNOWN:
            return STATE_UNKNOWN;
            break;
        case NM_DEVICE_STATE_UNMANAGED:
        case NM_DEVICE_STATE_UNAVAILABLE:
            return STATE_NA;
            break;
        case NM_DEVICE_STATE_DISCONNECTED:
            return STATE_STOPPED;
            break;
        case NM_DEVICE_STATE_PREPARE:
        case NM_DEVICE_STATE_CONFIG:
        case NM_DEVICE_STATE_NEED_AUTH:
        case NM_DEVICE_STATE_IP_CONFIG:
        case NM_DEVICE_STATE_IP_CHECK:
        case NM_DEVICE_STATE_SECONDARIES:
            return STATE_STARTING;
            break;
        case NM_DEVICE_STATE_ACTIVATED:
            return STATE_SERVICING;
            break;
        case NM_DEVICE_STATE_DEACTIVATING:
            return STATE_STOPPING;
            break;
        case NM_DEVICE_STATE_FAILED:
            return STATE_ABORTED;
            break;
    }
    return STATE_UNKNOWN;
}
