/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <CIM_LANEndpoint.h>
#include <CIM_ServiceAccessPoint.h>
#include <CIM_NetworkPort.h>
#include <CIM_IPAssignmentSettingData.h>
#include <Cura_IPProtocolEndpoint.h>
#include <CIM_ComputerSystem.h>
#include <Cura_IPConfigurationService.h>
#include <Cura_IPAssignmentSettingData.h>
#include <Cura_RemoteServiceAccessPoint.h>
#include <CIM_LogicalDevice.h>
#include "globals.h"
#include "port.h"
#include "endpoint.h"
#include "connection.h"

CMPIObjectPath *getComputerSystemOP(const CMPIBroker *cb, const char *ns)
{
    CIM_ComputerSystemRef computerSystemRef;
    CIM_ComputerSystemRef_Init(&computerSystemRef, cb, ns);
    CIM_ComputerSystemRef_Set_Name(&computerSystemRef, get_system_name());
    CIM_ComputerSystemRef_Set_CreationClassName(&computerSystemRef, get_system_creationg_class_name());

    CMPIStatus rc;
    // Small hack - we need to adjust class name, because it won't get found without it
    CMPIObjectPath *computerSystemOP = CIM_ComputerSystemRef_ToObjectPath(&computerSystemRef, &rc);
    computerSystemOP->ft->setClassName(computerSystemOP, get_system_creationg_class_name());
    return computerSystemOP;
}

CMPIObjectPath *getIPConfigurationSystemOP(const CMPIBroker *cb, const char *ns)
{
    Cura_IPConfigurationServiceRef ipConfigurationServiceRef;
    Cura_IPConfigurationServiceRef_Init(&ipConfigurationServiceRef, cb, ns);
    Cura_IPConfigurationServiceRef_Set_CreationClassName(&ipConfigurationServiceRef, "Cura_IPConfigurationService");
    Cura_IPConfigurationServiceRef_Set_Name(&ipConfigurationServiceRef, "IPConfigurationService");
    Cura_IPConfigurationServiceRef_Set_SystemCreationClassName(&ipConfigurationServiceRef, get_system_creationg_class_name());
    Cura_IPConfigurationServiceRef_Set_SystemName(&ipConfigurationServiceRef, get_system_name());

    CMPIStatus rc;
    return Cura_IPConfigurationServiceRef_ToObjectPath(&ipConfigurationServiceRef, &rc);
}

CMPIObjectPath *portToCIM_LANEndpointRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_LANEndpointRef lanEndpointRef;
    CIM_LANEndpointRef_Init(&lanEndpointRef, cb, ns);
    CIM_LANEndpointRef_Set_CreationClassName(&lanEndpointRef, "Cura_LANEndpoint");
    CIM_LANEndpointRef_Set_Name(&lanEndpointRef, port_get_id(port));
    CIM_LANEndpointRef_Set_SystemCreationClassName(&lanEndpointRef, get_system_creationg_class_name());
    CIM_LANEndpointRef_Set_SystemName(&lanEndpointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *lanEndpointOP = CIM_LANEndpointRef_ToObjectPath(&lanEndpointRef, &rc);
    lanEndpointOP->ft->setClassName(lanEndpointOP, "Cura_LANEndpoint");
    return lanEndpointOP;
}

CMPIObjectPath *portToCIM_NetworkPortRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_NetworkPortRef networkPortRef;
    CIM_NetworkPortRef_Init(&networkPortRef, cb, ns);
    CIM_NetworkPortRef_Set_CreationClassName(&networkPortRef, "Cura_EthernetPort");
    CIM_NetworkPortRef_Set_DeviceID(&networkPortRef, port_get_id(port));
    CIM_NetworkPortRef_Set_SystemCreationClassName(&networkPortRef, get_system_creationg_class_name());
    CIM_NetworkPortRef_Set_SystemName(&networkPortRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *networkPortOP = CIM_NetworkPortRef_ToObjectPath(&networkPortRef, &rc);
    networkPortOP->ft->setClassName(networkPortOP, "Cura_EthernetPort");
    return networkPortOP;
}

CMPIObjectPath *portToCIM_ServiceAccessPointRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_ServiceAccessPointRef serviceAccessPointRef;
    CIM_ServiceAccessPointRef_Init(&serviceAccessPointRef, cb, ns);
    CIM_ServiceAccessPointRef_Set_CreationClassName(&serviceAccessPointRef, "Cura_LANEndpoint");
    CIM_ServiceAccessPointRef_Set_Name(&serviceAccessPointRef, port_get_id(port));
    CIM_ServiceAccessPointRef_Set_SystemCreationClassName(&serviceAccessPointRef, get_system_creationg_class_name());
    CIM_ServiceAccessPointRef_Set_SystemName(&serviceAccessPointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *serviceAccessPointOP = CIM_ServiceAccessPointRef_ToObjectPath(&serviceAccessPointRef, &rc);
    serviceAccessPointOP->ft->setClassName(serviceAccessPointOP, "Cura_LANEndpoint");
    return serviceAccessPointOP;
}

CMPIObjectPath *portToCIM_LogicalDeviceOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_LogicalDeviceRef logicalDeviceRef;
    CIM_LogicalDeviceRef_Init(&logicalDeviceRef, cb, ns);
    CIM_LogicalDeviceRef_Set_CreationClassName(&logicalDeviceRef, "Cura_EthernetPort");
    CIM_LogicalDeviceRef_Set_DeviceID(&logicalDeviceRef, port_get_id(port));
    CIM_LogicalDeviceRef_Set_SystemCreationClassName(&logicalDeviceRef, get_system_creationg_class_name());
    CIM_LogicalDeviceRef_Set_SystemName(&logicalDeviceRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *logicalDeviceOP = CIM_LogicalDeviceRef_ToObjectPath(&logicalDeviceRef, &rc);
    logicalDeviceOP->ft->setClassName(logicalDeviceOP, "Cura_EthernetPort");
    return logicalDeviceOP;
}

CMPIObjectPath *endpointToCura_RemoteServiceAccessPointOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    Cura_RemoteServiceAccessPointRef remoteServiceAccessPointRef;
    Cura_RemoteServiceAccessPointRef_Init(&remoteServiceAccessPointRef, cb, ns);
    Cura_RemoteServiceAccessPointRef_Set_CreationClassName(&remoteServiceAccessPointRef, "Cura_RemoteServiceAccessPoint");
    Cura_RemoteServiceAccessPointRef_Set_Name(&remoteServiceAccessPointRef, endpoint_get_name(endpoint));
    Cura_RemoteServiceAccessPointRef_Set_SystemCreationClassName(&remoteServiceAccessPointRef, get_system_creationg_class_name());
    Cura_RemoteServiceAccessPointRef_Set_SystemName(&remoteServiceAccessPointRef, get_system_name());

    CMPIStatus rc;
    return Cura_RemoteServiceAccessPointRef_ToObjectPath(&remoteServiceAccessPointRef, &rc);
}

CMPIObjectPath *endpointToCIM_ServiceAccessPointRefOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    CIM_ServiceAccessPointRef serviceAccessPointRef;
    CIM_ServiceAccessPointRef_Init(&serviceAccessPointRef, cb, ns);
    CIM_ServiceAccessPointRef_Set_CreationClassName(&serviceAccessPointRef, "Cura_IPProtocolEndpoint");
    CIM_ServiceAccessPointRef_Set_Name(&serviceAccessPointRef, endpoint_get_name(endpoint));
    CIM_ServiceAccessPointRef_Set_SystemCreationClassName(&serviceAccessPointRef, get_system_creationg_class_name());
    CIM_ServiceAccessPointRef_Set_SystemName(&serviceAccessPointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *serviceAccessPointOP = CIM_ServiceAccessPointRef_ToObjectPath(&serviceAccessPointRef, &rc);
    serviceAccessPointOP->ft->setClassName(serviceAccessPointOP, "Cura_IPProtocolEndpoint");
    return serviceAccessPointOP;
}

CMPIObjectPath *endpointToCura_IPProtocolEndpointRefOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    Cura_IPProtocolEndpointRef ipProtocolEndpointRef;
    Cura_IPProtocolEndpointRef_Init(&ipProtocolEndpointRef, cb, ns);
    Cura_IPProtocolEndpointRef_Set_CreationClassName(&ipProtocolEndpointRef, "Cura_IPProtocolEndpoint");
    Cura_IPProtocolEndpointRef_Set_Name(&ipProtocolEndpointRef, endpoint_get_name(endpoint));
    Cura_IPProtocolEndpointRef_Set_SystemCreationClassName(&ipProtocolEndpointRef, get_system_creationg_class_name());
    Cura_IPProtocolEndpointRef_Set_SystemName(&ipProtocolEndpointRef, get_system_name());

    CMPIStatus rc;
    return Cura_IPProtocolEndpointRef_ToObjectPath(&ipProtocolEndpointRef, &rc);
}

CMPIObjectPath *connectionToCIM_IPAssignmentSettingDataRefOP(const Connection *connection, const CMPIBroker *cb, const char *ns)
{
    CIM_IPAssignmentSettingDataRef settingDataRef_generic;
    CIM_IPAssignmentSettingDataRef_Init(&settingDataRef_generic, cb, ns);
    CIM_IPAssignmentSettingDataRef_Set_InstanceID(&settingDataRef_generic, connection_get_id(connection));

    CMPIStatus rc;
    CMPIObjectPath *settingDataOP = CIM_IPAssignmentSettingDataRef_ToObjectPath(&settingDataRef_generic, &rc);
    settingDataOP->ft->setClassName(settingDataOP, "Cura_IPAssignmentSettingData");
    return settingDataOP;

}

CMPIObjectPath *settingToCura_IPAssignmentSettingDataSubclassOP(const Setting *setting, const CMPIBroker *cb, const char *ns)
{
    Cura_IPAssignmentSettingDataRef ipAssignmentSettingDataRef;
    Cura_IPAssignmentSettingDataRef_Init(&ipAssignmentSettingDataRef, cb, ns);
    const char *className;
    switch (setting_get_type(setting)) {
        case SETTING_STATIC:
            className = "Cura_StaticIPAssignmentSettingData";
            break;
        case SETTING_LINK_LOCAL:
            className = "Cura_StaticIPAssignmentSettingData";
            break;
        case SETTING_DHCP:
            className = "Cura_DHCPSettingData";
            break;
        default:
            warn("Unknown setting type: %d", setting_get_type(setting));
            break;
    }

    CMPIStatus rc;
    Cura_IPAssignmentSettingDataRef_Set_InstanceID(&ipAssignmentSettingDataRef, setting_get_id(setting));
    CMPIObjectPath *ipAssignmentSettingDataOP = Cura_IPAssignmentSettingDataRef_ToObjectPath(&ipAssignmentSettingDataRef, &rc);
    ipAssignmentSettingDataOP->ft->setClassName(ipAssignmentSettingDataOP, className);
    return ipAssignmentSettingDataOP;
}
