/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "setting_private.h"
#ifndef TESTING
#include "setting_nm.h"
#include <nm-setting-connection.h>
#include <nm-setting-ip4-config.h>
#include <nm-setting-ip6-config.h>
#endif
#include <string.h>


Setting *setting_new_link_local(Connection *connection, enum ProtocolType protocol)
{
    Setting *setting = setting_new(protocol, SETTING_LINK_LOCAL);
    asprintf(&setting->id, "%s_ipv%d", connection_get_id(connection), (protocol == IPv4) ? 4 : 6);
    asprintf(&setting->caption, "%s IPv%d", connection_get_name(connection), (protocol == IPv4) ? 4 : 6);
    return setting;
}

Setting *setting_new_DHCP(Connection *connection, enum ProtocolType protocol)
{
    Setting *setting = setting_new(protocol, SETTING_DHCP);
    asprintf(&setting->id, "%s_ipv%d_dhcp", connection_get_id(connection), (protocol == IPv4) ? 4 : 6);
    asprintf(&setting->caption, "%s IPv%d DHCP", connection_get_name(connection), (protocol == IPv4) ? 4 : 6);
    return setting;
}

Setting *setting_new_from_static_ipv4(Connection *connection, NMIP4Address *ipv4address, int index)
{
    Setting *setting = setting_new(IPv4, SETTING_STATIC);
    asprintf(&setting->id, "%s_ipv4_%d", connection_get_id(connection), index);
    asprintf(&setting->caption, "%s IPv4 %d", connection_get_name(connection), index);
    setting->address = ip4ToString(nm_ip4_address_get_address(ipv4address));
    setting->prefix = nm_ip4_address_get_prefix(ipv4address);
    setting->default_gateway = ip4ToString(nm_ip4_address_get_gateway(ipv4address));
    return setting;
}

Setting *setting_new_from_static_ipv6(Connection *connection, NMIP6Address *ipv6address, int index)
{
    Setting *setting = setting_new(IPv6, SETTING_STATIC);
    asprintf(&setting->id, "%s_ipv6_%d", connection_get_id(connection), index);
    asprintf(&setting->caption, "%s IPv6 %d", connection_get_name(connection), index);
    setting->address = ip6ToString(nm_ip6_address_get_address(ipv6address));
    setting->prefix = nm_ip6_address_get_prefix(ipv6address);
    setting->default_gateway = ip6ToString(nm_ip6_address_get_gateway(ipv6address));
    return setting;
}

const char *setting_get_nm_method(Setting *setting)
{
    switch (setting->type) {
        case SETTING_STATIC:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_MANUAL :
                    NM_SETTING_IP6_CONFIG_METHOD_MANUAL;
            break;
        case SETTING_DHCP:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_AUTO :
                    NM_SETTING_IP6_CONFIG_METHOD_AUTO;
            break;
        case SETTING_LINK_LOCAL:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_LINK_LOCAL :
                    NM_SETTING_IP6_CONFIG_METHOD_LINK_LOCAL;
            break;
        default:
            return (setting->protocol_type == IPv4) ?
                    NM_SETTING_IP4_CONFIG_METHOD_DISABLED :
                    NM_SETTING_IP6_CONFIG_METHOD_IGNORE;
    }
}
